-- Based on functions from Minimalist

TweakHub.Merchant = TweakHub:NewModule("Merchant", "AceEvent-3.0")

local L = LibStub("AceLocale-3.0"):GetLocale("TweakHubMerchant")
local inf = 1/0
local COPPER_ABBR = "c"
local SILVER_ABBR = "s"
local GOLD_ABBR = "g"

local COLOR_WHITE = "ffffff"
local COLOR_GREEN = "00ff00"
local COLOR_RED = "ff0000"
local COLOR_COPPER = "eda55f"
local COLOR_SILVER = "c7c7cf"
local COLOR_GOLD = "ffd700"

local GOLD, SILVER, COPPER = GOLD, SILVER, COPPER

--Credit to Sid67 (BasicBrokers author)
StaticPopupDialogs["TWEAKHUB_MERCHANT"] = {
	text = "Repair damaged equipment for %s?",
	button1 = "Repair",
	button2 = CANCEL,
	OnAccept = function()
		RepairAllItems(CanGuildBankRepair());
	end,
	timeout = 0,
	whileDead = 0,
	hideOnEscape = 1,
	cancels = "BASICBROKER_DURABILITY",
}

if not COPPER and COPPER_AMOUNT then
	GOLD = GOLD_AMOUNT:gsub("%s*%%d%s*", "")
	SILVER = SILVER_AMOUNT:gsub("%s*%%d%s*", "")
	COPPER = COPPER_AMOUNT:gsub("%s*%%d%s*", "")
end

TweakHub.Merchant.Name = L["Merchant & Gossip Tweaks"]

function TweakHub.Merchant:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("TweakHubMerchantDB", self:GetDefaults(), "Default")
end

function TweakHub.Merchant:OnEnable()

	self:DoMerchant()
	self:DoGossip()
	self:DoTaxi()

end

function TweakHub.Merchant:DoMerchant()
	if self.db.profile.AutoRepair or self.db.profile.RepairRemind or self.db.profile.AutoSell or self.db.profile.Scroll then
		self:RegisterEvent("MERCHANT_SHOW")
		self:RegisterEvent("MERCHANT_CLOSED")
	else
		self:UnregisterEvent("MERCHANT_SHOW")
		self:UnregisterEvent("MERCHANT_CLOSED")
	end
end

function TweakHub.Merchant:DoGossip()
	if self.db.profile.SkipGossip then
		self:RegisterEvent("GOSSIP_SHOW")
	else
		self:UnregisterEvent("GOSSIP_SHOW")
	end
end

function TweakHub.Merchant:DoTaxi()
	if self.db.profile.AutoDismount then
		self:RegisterEvent("TAXIMAP_OPENED")
	else
		self:UnregisterEvent("TAXIMAP_OPENED")
	end
end

function TweakHub.Merchant:DoScrolling()
	local numMerchantItems = GetMerchantNumItems()
	if (numMerchantItems ~= nil and numMerchantItems > MERCHANT_ITEMS_PER_PAGE) then
		if (arg1) and (arg1 < 0) then
			if (not (MerchantFrame.page == ceil(numMerchantItems / MERCHANT_ITEMS_PER_PAGE))) then
				MerchantNextPageButton_OnClick()
			end
		else
			if (MerchantFrame.page > 1) then
				MerchantPrevPageButton_OnClick()
			end
		end
	end
end

function TweakHub.Merchant:MERCHANT_SHOW()
	if self.db.profile.AutoSell then
		local bag, slot

		for bag = 0, 4 do
			if GetContainerNumSlots(bag) > 0 then
				for slot = 1, GetContainerNumSlots(bag) do
					local _, _, _, quality = GetContainerItemInfo(bag, slot)

					if (quality == 0 or quality == -1) then
						if (self:IsJunk(GetContainerItemLink(bag, slot))) then
							UseContainerItem(bag, slot)
						end
					end
				end
			end
		end
	end

	if self.db.profile.AutoRepair then
		local cost = GetRepairAllCost()
		local money = GetMoney()

		if money < cost then
			TweakHub:Print(L["Insufficient Funds to Repair"])
		elseif money >= cost and cost > 0 then
			RepairAllItems(CanGuildBankRepair());
		end
	end

	if self.db.profile.RepairRemind then
		local cost = GetRepairAllCost()
		local money = GetMoney()

		if money < cost then
			TweakHub:Print(L["Insufficient Funds to Repair"])
		elseif money >= cost and cost > 0 then
			StaticPopup_Show ("TWEAKHUB_MERCHANT", self:FormatMoneyExtended(cost))
		end
	end

	if self.db.profile.Scroll then
		if MerchantFrame:IsVisible() then
			MerchantFrame:EnableMouseWheel(true)
			MerchantFrame:SetScript("OnMouseWheel", self.DoScrolling)
		end
	end
end

function TweakHub.Merchant:MERCHANT_CLOSED()
	StaticPopup_Hide("TWEAKHUB_MERCHANT")
end

function TweakHub.Merchant:GOSSIP_SHOW()
	local bwlText = L["The orb's markings match the brand on your hand."]
	local mcText = L["You see large cavernous tunnels"]
	local text = GetGossipText()

	if text == bwlText or strsub(text, 1, 31) == mcText then
		SelectGossipOption(1)
	else
		local gossipOptions = { GetGossipOptions() }
		for i = 2, getn(gossipOptions), 2 do
			if gossipOptions[i] == "taxi" or gossipOptions[i] == "battlemaster" or gossipOptions[i] == "banker" then
				SelectGossipOption(i / 2)
			end
		end
	end
end

function TweakHub.Merchant:TAXIMAP_OPENED()
	Dismount()
end

function TweakHub.Merchant:IsJunk(link)
	for color, name in string.gmatch(link, "(|c%x+)|Hitem:.+|h%[(.-)%]|h|r") do
		if color == ITEM_QUALITY_COLORS[0].hex then
			return true
		else
			return false
		end
	end
end

function TweakHub.Merchant:FormatMoneyExtended(value, colorize, textColor)
	local gold = abs(value / 10000)
	local silver = abs(mod(value / 100, 100))
	local copper = abs(mod(value, 100))

	local negl = ""
	local color = COLOR_WHITE
	if value > 0 then
		if textColor then
			color = COLOR_GREEN
		end
	elseif value < 0 then
		negl = "-"
		if textColor then
			color = COLOR_RED
		end
	end
	if colorize then
		if value == inf or value == -inf then
			return format("|cff%s%s|r", color, value)
		elseif value ~= value then
			return format("|cff%s0|r|cff%s %s|r", COLOR_WHITE, COLOR_COPPER, COPPER)
		elseif value >= 10000 or value <= -10000 then
			return format("|cff%s%s%d|r|cff%s %s|r |cff%s%d|r|cff%s %s|r |cff%s%d|r|cff%s %s|r", color, negl, gold, COLOR_GOLD, GOLD, color, silver, COLOR_SILVER, SILVER, color, copper, COLOR_COPPER, COPPER)
		elseif value >= 100 or value <= -100 then
			return format("|cff%s%s%d|r|cff%s %s|r |cff%s%d|r|cff%s %s|r", color, negl, silver, COLOR_SILVER, SILVER, color, copper, COLOR_COPPER, COPPER)
		else
			return format("|cff%s%s%d|r|cff%s %s|r", color, negl, copper, COLOR_COPPER, COPPER)
		end
	else
		if value == inf or value == -inf then
			return format("%s", value)
		elseif value ~= value then
			return format("0 %s", COPPER)
		elseif value >= 10000 or value <= -10000 then
			return format("%s%d %s %d %s %d %s", negl, gold, GOLD, silver, SILVER, copper, COPPER)
		elseif value >= 100 or value <= -100 then
			return format("%s%d %s %d %s", negl, silver, SILVER, copper, COPPER)
		else
			return format("%s%d %s", negl, copper, COPPER)
		end
	end
end

